﻿using Processor;

namespace UI_WinForms.Components;

public partial class Translations : UserControl
{
    private readonly Dictionary<string, Translation> translationComponents = new();

    public Translations()
    {
        this.InitializeComponent();
        this.Load += async (sender, args) => await this.SetupTranslations();
        AppEvents.WhenTextElementChanged += async (sender, textElement) =>
        {
            var allTranslations = await TranslationProcessor.GetTranslations(textElement);
            foreach (var translation in allTranslations)
                if (this.translationComponents.ContainsKey(translation.Culture))
                    await this.translationComponents[translation.Culture].Configure(translation);
        };
    }

    private async Task SetupTranslations()
    {
        if(this.DesignMode)
            return;

        // Apply a custom order for the cultures. We want to show the default culture first,
        // then the other cultures in order of their index. Please note: whatever we want to
        // show as first, we have to return last!
        async IAsyncEnumerable<AppSettings.CultureInfo> DesiredOrder()
        {
            // Get all configured cultures:
            var cultureInfos = (await AppSettings.GetCultureInfos()).OrderByDescending(n => n.Index).ToList();
            
            // Get the main culture:
            var mainCultureIndex = await AppSettings.GetDeepLSourceCultureIndex();
            
            // First, we return all the non-main cultures in descending order of their index:
            foreach (var cultureInfo in cultureInfos.Where(n => n.Index != mainCultureIndex))
                yield return cultureInfo;
            
            // Last, we return the main culture:
            if(cultureInfos.Any(n => n.Index == mainCultureIndex))
                yield return cultureInfos.First(n => n.Index == mainCultureIndex);
        }

        await foreach (var cultureInfo in DesiredOrder())
        {
            var translationComponent = new Translation(cultureInfo);
            this.translationComponents.Add(cultureInfo.Code, translationComponent);
            this.panelTranslations.Controls.Add(translationComponent);
        }
    }
}